from ..abstract import ErdReadOnlyConverter
from ..primitives import *
from gekitchensdk.erd.values.oven import OvenConfiguration, ErdOvenConfiguration

class OvenConfigurationConverter(ErdReadOnlyConverter[OvenConfiguration]):
    def erd_decode(self, value: str) -> OvenConfiguration:
        if not value:
            n = 0
        else:
            n = erd_decode_int(value)

        config = OvenConfiguration(
            has_knob=bool(n & ErdOvenConfiguration.HAS_KNOB.value),
            has_warming_drawer=bool(n & ErdOvenConfiguration.HAS_WARMING_DRAWER.value),
            has_light_bar=bool(n & ErdOvenConfiguration.HAS_LIGHT_BAR.value),
            has_lower_oven=bool(n & ErdOvenConfiguration.HAS_LOWER_OVEN.value),
            has_lower_oven_kitchen_timer=bool(n & ErdOvenConfiguration.HAS_LOWER_OVEN_KITCHEN_TIMER.value),
            raw_value=value,
        )
        return config



import logging

from gekitchensdk.erd.converters.abstract import ErdReadOnlyConverter
from gekitchensdk.erd.converters.primitives import *
from gekitchensdk.erd.values.dishwasher import ErdOperatingMode, OperatingMode, OPERATING_MODE_MAP

_LOGGER = logging.getLogger(__name__)

class OperatingModeConverter(ErdReadOnlyConverter[OperatingMode]):
    def erd_decode(self, value: str) -> OperatingMode:
        """Decode the dishwasher operating state """
        try:
            om = ErdOperatingMode(value)
            _LOGGER.debug(f'raw operating mode value: {om}')
            return OPERATING_MODE_MAP[om]
        except (KeyError, ValueError):
            return ErdOperatingMode.NA



from .erd_appliance_type_converter import ErdApplianceTypeConverter
from .erd_clock_format_converter import ErdClockFormatConverter
from .erd_end_tone_converter import ErdEndToneConverter
from .erd_measurement_units_converter import ErdMeasurementUnitsConverter
from .erd_model_serial_converter import ErdModelSerialConverter
from .erd_software_version_converter import ErdSoftwareVersionConverter
from .erd_sound_level_converter import ErdSoundLevelConverter



import enum

#I might be doing something wrong, but I saw an "02" value come through
#which doesn't seem to be in any of the documentation I saw... it registered
#as low on the interface, so I'll just assume there's two codes
@enum.unique
class ErdRinseAgentRaw(enum.Enum):
    RINSE_AGENT_GOOD = "00"
    RINSE_AGENT_LOW1 = "01"
    RINSE_AGENT_LOW2 = "02"

@enum.unique
class ErdRinseAgent(enum.Enum):
    NA = "FF"
    RINSE_AGENT_GOOD = "00"
    RINSE_AGENT_LOW = "01"

    def stringify(self, **kwargs):
        return self.name.replace("RINSE_AGENT_","").replace("_"," ").title()




from textwrap import wrap

from ..abstract import ErdReadOnlyConverter
from ..primitives import *

class ErdSoftwareVersionConverter(ErdReadOnlyConverter[str]):
    def erd_decode(self, value) -> str:
        """
        Decode a software version string.
        These are sent as four bytes, encoding each part of a four-element version string.
        """
        vals = wrap(value, 2)
        return '.'.join(str(erd_decode_int(val)) for val in vals)



from ..abstract import ErdReadWriteConverter
from ..primitives import *

from gekitchensdk.erd.values import ErdMeasurementUnits

class ErdMeasurementUnitsConverter(ErdReadWriteConverter[ErdMeasurementUnits]):
    def erd_decode(self, value: str) -> ErdMeasurementUnits:
        return ErdMeasurementUnits(int(value))
    def erd_encode(self, value: ErdMeasurementUnits) -> str:
        return f'{value.value:02d}'



from .erd_available_cook_mode import ErdAvailableCookMode
from .erd_cooktop_config import ErdCooktopConfig
from .erd_cooktop_status import ErdCooktopStatus
from .erd_oven_configuration import ErdOvenConfiguration
from .erd_oven_cook_mode import ErdOvenCookMode
from .erd_oven_state import ErdOvenState

from .available_cook_mode import AvailableCookMode
from .oven_configuration import OvenConfiguration
from .oven_cook_mode import OvenCookMode
from .oven_cook_mode_mapping import OVEN_COOK_MODE_MAP
from .oven_cook_setting import OvenCookSetting
from .oven_ranges import OvenRanges
from .burner import Burner
from .cooktop_status import CooktopStatus
from .erd_precision_cooking_probe_battery_status import ErdPrecisionCookingProbeBatteryStatus
from .erd_precision_cooking_start_sous_vide_timer_active_status import ErdPrecisionCookingStartSousVideTimerActiveStatus
from .erd_precision_cooking_probe_target_time_reached import ErdPrecisionCookingProbeTargetTimeReached
from .erd_precision_cooking_app_probe_control_mode import ErdPrecisionCookingAppProbeControlMode



STATE_OVEN_BAKE = "Bake"
STATE_OVEN_BAKE_TWO_TEMP = "Bake (Two Temp.)"
STATE_OVEN_BAKED_GOODS = "Baked Goods"
STATE_OVEN_BROIL_HIGH = "Broil (High)"
STATE_OVEN_BROIL_LOW = "Broil (Low)"
STATE_OVEN_CONV_BAKE = "Convection Bake"
STATE_OVEN_CONV_BAKE_TWO_TEMP = "Convection Bake (Two Temp.)"
STATE_OVEN_CONV_BROIL_CRISP = "Convection Broil (Crisp)"
STATE_OVEN_CONV_BROIL_HIGH = "Convection Broil (High)"
STATE_OVEN_CONV_BROIL_LOW = "Convection Broil (Low)"
STATE_OVEN_CONV_BAKE_MULTI = "Convection Multi Bake"
STATE_OVEN_CONV_ROAST = "Convection Roast"
STATE_OVEN_CONV_ROAST_TWO_TEMP = "Convection Roast (Two Temp.)"
STATE_OVEN_DUAL_BROIL_HIGH = "Dual Broil (High)"
STATE_OVEN_DUAL_BROIL_LOW = "Dual Broil (Low)"
STATE_OVEN_DELAY = "Delayed Start"
STATE_OVEN_FROZEN_PIZZA = "Frozen Pizza"
STATE_OVEN_FROZEN_SNACKS = "Frozen Snacks"
STATE_OVEN_MULTI_BAKE = "Multi Bake"
STATE_OVEN_PREHEAT = "Preheat"
STATE_OVEN_PROBE = "Probe"
STATE_OVEN_PROOF = "Proof"
STATE_OVEN_OFF = "Off"
STATE_OVEN_SABBATH = "Sabbath Mode"
STATE_OVEN_SELF_CLEAN = "Self Clean"
STATE_OVEN_SPECIAL = "Special"
STATE_OVEN_STEAM_CLEAN = "Steam Clean"
STATE_OVEN_TIMED = "Timed"
STATE_OVEN_UNKNOWN = "Unknown"
STATE_OVEN_WARM = "Keep Warm"



from .erd_operating_mode import ErdOperatingMode
from .operating_mode import OperatingMode

OPERATING_MODE_MAP = {
    ErdOperatingMode.NA: OperatingMode.STATUS_DASH,
    ErdOperatingMode.PAUSE: OperatingMode.STATUS_PAUSED,
    ErdOperatingMode.LOW_POWER: OperatingMode.STATUS_OFF,
    ErdOperatingMode.POWER_UP: OperatingMode.STATUS_OFF,
    ErdOperatingMode.STANDBY: OperatingMode.STATUS_OFF,
    ErdOperatingMode.DELAY_START: OperatingMode.STATUS_DELAY,
    ErdOperatingMode.CYCLE_ACTIVE: OperatingMode.STATUS_CYCLE_ACTIVE,
    ErdOperatingMode.EOC: OperatingMode.STATUS_OFF,
    ErdOperatingMode.DOWNLOAD_MODE: OperatingMode.STATUS_OFF,
    ErdOperatingMode.SENSOR_CHECK_MODE: OperatingMode.STATUS_OFF,
    ErdOperatingMode.LOAD_ACTIVATION_MODE: OperatingMode.STATUS_OFF,
    ErdOperatingMode.MC_ONLY_MODE: OperatingMode.STATUS_OFF,
    ErdOperatingMode.WARNING_MODE: OperatingMode.STATUS_OFF,
    ErdOperatingMode.CONTROL_LOCKED: OperatingMode.CONTROL_LOCKED,
    ErdOperatingMode.CSM_TRIPPED: OperatingMode.STATUS_OFF
}



import enum

@enum.unique
class ErdCycleStateRaw(enum.Enum):
    STATE_01 = "01"
    STATE_02 = "02"
    STATE_03 = "03"
    STATE_04 = "04"
    STATE_05 = "05"
    STATE_06 = "06"
    STATE_07 = "07"
    STATE_08 = "08"
    STATE_09 = "09"
    STATE_10 = "10"
    STATE_11 = "11"
    STATE_12 = "12"
    STATE_13 = "13"
    STATE_14 = "14"
    STATE_15 = "15"
    STATE_16 = "16"
    STATE_17 = "17"
    STATE_18 = "18"

@enum.unique
class ErdCycleState(enum.Enum):
    NA = -1
    PRE_WASH = 1
    SENSING = 2
    MAIN_WASH = 3
    DRYING = 4
    SANITIZING = 5
    RINSING = 6
    PAUSE = 7



from .erd_appliance_type import ErdApplianceType
from .erd_clock_format import ErdClockFormat
from .erd_end_tone import ErdEndTone
from .erd_measurement_units import ErdMeasurementUnits
from .erd_on_off import ErdOnOff
from .erd_present import ErdPresent
from .erd_sound_level import ErdSoundLevel
